//-----------------------------------------------------------------------------
// name: HelloSine.cpp
// desc: hello sine wave, real-time
//
// author: Ge Wang (ge@ccrma.stanford.edu)
//   date: fall 2011
//   uses: RtAudio by Gary Scavone
//-----------------------------------------------------------------------------
#include "RtAudio.h"
#include "chuck_fft.h"
#include "biquadFilter.h"
#include "Firework.h"
#include <math.h>
#include <stdlib.h>
#include <iostream>
using namespace std;

#ifdef __MACOSX_CORE__
#include <GLUT/glut.h>
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#else
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#endif




//-----------------------------------------------------------------------------
// function prototypes
//-----------------------------------------------------------------------------
void initGfx();
void idleFunc();
void displayFunc();
void reshapeFunc( GLsizei width, GLsizei height );
void keyboardFunc( unsigned char, int, int );
void mouseFunc( int button, int state, int x, int y );

// our datetype
#define SAMPLE float
// our fft datatype
#define F_SAMPLE complex
// corresponding format for RtAudio
#define MY_FORMAT RTAUDIO_FLOAT32
// sample rate
#define MY_SRATE 44100
// number of channels
#define MY_CHANNELS 1
// for convenience
#define MY_PIE 3.14159265358979

// width and height
long g_width = 1024;
long g_height = 720;
// global buffer
SAMPLE * g_buffer = NULL;
long g_bufferSize;
// global fft buffer
SAMPLE * g_fbuffer = NULL;
long g_fbufferSize;
// global window buffer
SAMPLE* g_window;
long g_windowSize;
// global lowpass buffer
SAMPLE* g_lowpassBuffer;
long g_lowpassBufferSize;
//globle realtime signal scale
float g_xScale = 2;
//globla number of Spirals
int g_numberOfSpiral = 12;
//saving buffer
complex** saveBuffer;
//global number of saving buffer
int numberOfSavingBuffer = g_numberOfSpiral;
//globla time index
int time_index = 0;
//Mode index choose between spectrum or firework mode
int program_mode = 0;
//initialize number of fireworks and particles in each firework
int numberOfFireworks = 0;
//Fire work
Firework firework[30];

SAMPLE Last_avgAmp = 0;

int g_time = 0;

GLfloat heartAlpha = 0.5;

int g_Tempo_Period = 0;

int beat = 0;

int heartRotate = 0;

int silent_time = 0;

int music_time = 0;

int musicState = 0;
//-----------------------------------------------------------------------------
// name: callme()
// desc: audio callback
//-----------------------------------------------------------------------------
int callme( void * outputBuffer, void * inputBuffer, unsigned int numFrames,
            double streamTime, RtAudioStreamStatus status, void * data )
{
    // cast!
    SAMPLE * input = (SAMPLE *)inputBuffer;
    SAMPLE * output = (SAMPLE *)outputBuffer;
    //memcpy to new buffer
    
    
    // fill
    for( int i = 0; i < numFrames; i++ )
    {
        // assume mono
        g_buffer[i]  = input[i];
        
        // zero output
        output[i] = 0;
    }
    
    //data(time domain) -> windowing -> fft -> spectrum -> scale
    //hanning(input,numFrames);
    
    return 0;
}

//-----------------------------------------------------------------------------
// Name: manual()
// Desc: Print user manual
//-----------------------------------------------------------------------------
void manual()
{
    cout << "----------------------------------------------------" << endl;
    cout << "PROGRAM-NAME:PotVisualizer (v1.0)" << endl;
    cout << "AUTHOR: Kai-Chieh(Jay) Huang" << endl;
    cout << "http://website/" << endl;
    cout << "----------------------------------------------------" << endl;
    cout << "'q' - quit visualization" << endl;
    cout << "'r' - rotate the screen in y-axes (CounterClockwise)" << endl;
    cout << "'e' - rotate the screen in y-axes (Clockwise)" << endl;
    cout << "(shift)'+' - increase number of spectrum spirals" << endl;
    cout << "(shift)'_' - decrease number of spectrum spirals" << endl;
    cout << "'c' - change between Beat Detect/Spectrum mode" << endl;
    cout << "'h' - rotate the background heart in Beat Detect mode" << endl;
    cout << "----------------------------------------------------" << endl;
}

//-----------------------------------------------------------------------------
// name: main()
// desc: entry point
//-----------------------------------------------------------------------------
int main( int argc, char ** argv )
{
    //Print user maual
    manual();
    
    // instantiate RtAudio object
    RtAudio audio;
    // variables
    unsigned int bufferBytes = 0;
    // frame size
    unsigned int bufferFrames = 512;
    
    // check for audio devices
    if( audio.getDeviceCount() < 1 )
    {
        // nopes
        cout << "no audio devices found!" << endl;
        exit( 1 );
    }

    // initialize GLUT
    glutInit( &argc, argv );
    // init gfx
    initGfx();
    
    // let RtAudio print messages to stderr.
    audio.showWarnings( true );

    // set input and output parameters
    RtAudio::StreamParameters iParams, oParams;
    iParams.deviceId = audio.getDefaultInputDevice();
    iParams.nChannels = MY_CHANNELS;
    iParams.firstChannel = 0;
    oParams.deviceId = audio.getDefaultOutputDevice();
    oParams.nChannels = MY_CHANNELS;
    oParams.firstChannel = 0;
    
    // create stream options
    RtAudio::StreamOptions options;

    // go for it
    try {
        // open a stream
        audio.openStream( &oParams, &iParams, MY_FORMAT, MY_SRATE, &bufferFrames, &callme, (void *)&bufferBytes, &options );
    }
    catch( RtAudioError& e )
    {
        // error!
        cout << e.getMessage() << endl;
        exit( 1 );
    }

    // compute
    bufferBytes = bufferFrames * MY_CHANNELS * sizeof(SAMPLE);
    // allocate global buffer
    g_bufferSize = bufferFrames;
    g_buffer = new SAMPLE[g_bufferSize];
    g_fbufferSize = g_bufferSize;
    g_fbuffer = new SAMPLE[g_fbufferSize];
    g_lowpassBufferSize = bufferFrames;
    g_lowpassBuffer = new SAMPLE[g_lowpassBufferSize];
    //For correctness, performance
    memset( g_buffer, 0, sizeof(SAMPLE)*g_bufferSize );
    memset( g_fbuffer, 0, sizeof(SAMPLE)*g_fbufferSize );
    memset( g_lowpassBuffer, 0, sizeof(SAMPLE)*g_lowpassBufferSize);
    
    //allocate buffer to hold window
    g_windowSize = bufferFrames;
    g_window = new SAMPLE[g_windowSize];
    //generate the window
    hanning(g_window,g_windowSize);
    
    //allocate an array of buffer to save previous fft results
    saveBuffer = new complex*[48];
    for (int i=0; i<48; i++)
        saveBuffer[i] = new complex[bufferFrames/2];
    
    
    // go for it
    try {
        // start stream
        audio.startStream();

        // let GLUT handle the current thread from here
        glutMainLoop();
        
        // stop the stream.
        audio.stopStream();
    }
    catch( RtAudioError& e )
    {
        // print error message
        cout << e.getMessage() << endl;
        goto cleanup;
    }
    
cleanup:
    // close if open
    if( audio.isStreamOpen() )
        audio.closeStream();
    
    // done
    return 0;
}



//-----------------------------------------------------------------------------
// Name: reshapeFunc( )
// Desc: called when window size changes
//-----------------------------------------------------------------------------
void initGfx()
{
    // double buffer, use rgb color, enable depth buffer
    glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH |GLUT_ACCUM);
    // initialize the window size
    glutInitWindowSize( g_width, g_height );
    // set the window postion
    glutInitWindowPosition( 100, 100 );
    // create the window
    glutCreateWindow( "PotVisualizer" );
    
    // set the idle function - called when idle
    glutIdleFunc( idleFunc );
    // set the display function - called when redrawing
    glutDisplayFunc( displayFunc );
    // set the reshape function - called when client area changes
    glutReshapeFunc( reshapeFunc );
    // set the keyboard function - called on keyboard events
    glutKeyboardFunc( keyboardFunc );
    // set the mouse function - called on mouse stuff
    glutMouseFunc( mouseFunc );
    
    // set clear color
    glClearColor( 0.05, 0.05, 0.05, 0.8 );
    // enable color material
    glEnable( GL_COLOR_MATERIAL );
    // enable depth test
    glEnable( GL_DEPTH_TEST );
    //// Enable blending (we need this to be able to use an alpha component)
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable( GL_POINT_SMOOTH );
    glClearAccum(0.0f, 0.0f, 0.0f, 1.0f);
}




//-----------------------------------------------------------------------------
// Name: reshapeFunc( )
// Desc: called when window size changes
//-----------------------------------------------------------------------------
void reshapeFunc( GLsizei w, GLsizei h )
{
    // save the new window size
    g_width = w; g_height = h;
    // map the view port to the client area
    glViewport( 0, 0, w, h );
    // set the matrix mode to project
    glMatrixMode( GL_PROJECTION );
    // load the identity matrix
    glLoadIdentity( );
    // create the viewing frustum
    gluPerspective( 45.0, (GLfloat) w / (GLfloat) h, 1.0, 300.0 );
    // set the matrix mode to modelview
    glMatrixMode( GL_MODELVIEW );
    // load the identity matrix
    glLoadIdentity( );
    // position the view point
    gluLookAt( 0.0f, 0.0f, 10.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f );
}




//-----------------------------------------------------------------------------
// Name: keyboardFunc( )
// Desc: key event
//-----------------------------------------------------------------------------
void keyboardFunc( unsigned char key, int x, int y )
{
    int mode = glutGetModifiers();
    
    if (mode==1)//ALT=4, SHIFT=1,  CTRL=2
    {
        switch (key)
        {
            case '+':
                g_numberOfSpiral += 1;
                break;
            case '_':
                g_numberOfSpiral -= 1;
                break;
        }
    }
    else if (mode==2)//ALT=4, SHIFT=1,  CTRL=2
    {
        switch (key)
        {
            case '+':
                g_xScale += 0.3;
                break;
            case '_':
                g_xScale -= 0.3;
                break;
        }
    }
    else
    {
        switch( key )
        {
            case 'r':
                glRotatef(1.5,0,1,0);
                break;
            case 'e':
                glRotatef(-1.5,0,1,0);
                break;
            case 'c':
                program_mode += 1;
                break;
            case 'q':
                exit(1);
                break;
            case 'h':
                heartRotate = 1;
                break;
        }
    }
    glutPostRedisplay( );
}




//-----------------------------------------------------------------------------
// Name: mouseFunc( )
// Desc: handles mouse stuff
//-----------------------------------------------------------------------------
void mouseFunc( int button, int state, int x, int y )
{
    if( button == GLUT_LEFT_BUTTON )
    {
        // when left mouse button is down
        if( state == GLUT_DOWN )
        {
        }
        else
        {
        }
    }
    else if ( button == GLUT_RIGHT_BUTTON )
    {
        // when right mouse button down
        if( state == GLUT_DOWN )
        {
        }
        else
        {
        }
    }
    else
    {
    }
    
    glutPostRedisplay( );
}




//-----------------------------------------------------------------------------
// Name: idleFunc( )
// Desc: callback from GLUT
//-----------------------------------------------------------------------------
void idleFunc( )
{
    // render the scene
    glutPostRedisplay( );
}


//-----------------------------------------------------------------------------
// Name: textGenerateFunc()
// Desc: use glutStrokeCharacter() to generate text on screen
//-----------------------------------------------------------------------------

void textGenerateFunc(GLfloat x,GLfloat y, void*font, char* text, float scale, float lineWidth)
{
   
    glPushMatrix();
    glTranslatef(x,y,0);
    //glColor3f( 0.2, 0.6, 1 );
    glColor3f( 1, 1, 0 );
    glLineWidth( lineWidth );
    glScalef(scale/152.38, scale/152.38, scale/152.38);
    for (char* string=text; *string!='\0'; string++)
    {
        glutStrokeCharacter(font,*string);
    }
    
    glPopMatrix();
}


//-----------------------------------------------------------------------------
// Name: drawStanfordFunc()
// Desc: use to generate a heart shape
//-----------------------------------------------------------------------------

void drawStanfordFunc()
{

    int x = -1.8, y = -1;
    glPushMatrix();
    
        heartAlpha = 0.5 + 0.5*sin(2*MY_PIE*g_time/100);
        glColor4f(1,0,0,heartAlpha);

        glLineWidth(5);
        glScalef(1.3,1.3,1.3);
        glBegin(GL_LINE_STRIP);
    
            glVertex2f(x,y);
            glVertex2f(x,y-0.5);
            glVertex2f(x+0.6,y-0.5*2);
            glVertex2f(x+(1+0.6),y-0.5*2);
            glVertex2f(x+(1+2*0.6),y-0.5);
            glVertex2f(x+(1+2*0.6),y+0.5*2);
            glVertex2f(x+(1+0.6),y+0.5*3);
            glVertex2f(x+0.6,y+0.5*3);
            glVertex2f(x+0.6,y+0.5*4);
            glVertex2f(x+(1+0.6),y+0.5*4);
            glVertex2f(x+(1+0.6),y+1.8);
            glVertex2f(x+(1+2*0.6),y+1.8);
            glVertex2f(x+(1+2*0.6),y+0.5*4);
            glVertex2f(x+(1+2*0.6),y+0.5*5);
            glVertex2f(x+(1+0.6),y+0.5*6);
            glVertex2f(x+0.6,y+0.5*6);
            glVertex2f(x,y+0.5*5);
            glVertex2f(x,y+0.5*2);
            glVertex2f(x+0.4,y+0.7);
            glVertex2f(x+(1+0.6),y+0.7);
            glVertex2f(x+(1+0.6),y-0.2);
            glVertex2f(x+0.6,y-0.2);
            glVertex2f(x+0.6,y);
            glVertex2f(x,y);
        glEnd();

    glPopMatrix();
     textGenerateFunc(-2.3+0.1*sin(2*MY_PIE*g_time/200),0.1*sin(2*MY_PIE*g_time/200),GLUT_STROKE_ROMAN,"MUSIC PLEASE!",0.8,5);
}

//-----------------------------------------------------------------------------
// Name: drawHeartFunc()
// Desc: use to generate a heart shape
//-----------------------------------------------------------------------------

void drawHeartFunc(int beat)
{
    glPushMatrix();
    
    if (beat==1)
        glScalef(1.05, 1.05, 1.05);
    
    heartAlpha = 0.5 + 0.5*sin(2*MY_PIE*g_time/100);
    glColor4f(1,0.08,0.4,heartAlpha);
    glLineWidth(5);
    glBegin(GL_LINE_STRIP);
        glVertex2f(2,0);
        glVertex2f(0,-2);
        glVertex2f(-2,0);
    glEnd();
    glBegin(GL_LINE_LOOP);
    
    for (int i=0; i<20; i++)
    {
        glVertex2f(-1+1.414*cos(MY_PIE/4+1.1*(i*MY_PIE/20)),1+1.414*sin(MY_PIE/4+1.1*(i*MY_PIE/20)));
    }
    for (int i=0; i<20; i++)
    {
        glVertex2f(1+1.414*cos(-MY_PIE/4+1.1*(i*MY_PIE/20)),1+1.414*sin(-MY_PIE/4+1.1*(i*MY_PIE/20)));
    }
    glEnd();
    textGenerateFunc(-0.8,1,GLUT_STROKE_ROMAN,"KEEP",0.7,4);
    textGenerateFunc(-0.8,0.46,GLUT_STROKE_ROMAN,"CALM",0.7,4);
    textGenerateFunc(-0.45,0.05,GLUT_STROKE_ROMAN,"AND",0.4,4);
    textGenerateFunc(-0.9,-0.66,GLUT_STROKE_ROMAN,"ROCK",0.8,4);
    textGenerateFunc(-0.5,-1.2,GLUT_STROKE_ROMAN,"ON",0.7,4);

    if (beat==1)
        glScalef(1/1.05, 1/1.05, 1/1.05);

    glPopMatrix();
}

//-----------------------------------------------------------------------------
// Name: spiralGenerateFunc()
// Desc: use to generate a circle of rainbow spirals
//-----------------------------------------------------------------------------
void spiralGenerateFunc(int numberOfSpiral, complex* cbuf)
{
    //Draw a circle of rainbow spirals
    glPushMatrix();
    
    GLfloat x_Cir = 0;
    GLfloat y_Cir = 0;
    GLfloat z_Cir = 0;
    float   radius = .7;
    float   radiusLocation = 3.3;
    float   angle = 0;
    float   AngleLocation = 0;
    
    
    glLineWidth(5);
    
    
        for (int j=0; j<numberOfSpiral; j++)
        {
            
            x_Cir = radiusLocation*cos(AngleLocation);
            y_Cir = radiusLocation*sin(AngleLocation);
            
            glBegin(GL_LINE_LOOP);
            
            for (int i=0; i<100; i++)
            {
                if (i<14)
                    glColor3f(1, 0, 0);
                else if(i<28)
                    glColor3f(1, 0.5, 0);
                else if(i<42)
                    glColor3f(1, 1, 0);
                else if(i<56)
                    glColor3f(0, 1, 0);
                else if(i<70)
                    glColor3f(0, 0, 1);
                else if(i<84)
                    glColor3f(0.4, 0, 0.6);
                else
                    glColor3f(0.6, 0, 1);
                if (j==0)
                    glVertex3f(x_Cir+cos(angle)*(radius+sqrt(25*cmp_abs(cbuf[i]))),y_Cir+sin(angle)*(radius+sqrt(25*cmp_abs(cbuf[i]))),z_Cir);
                else
                    glVertex3f(x_Cir+cos(angle)*(radius+sqrt(25*cmp_abs(saveBuffer[j-1][i]))),y_Cir+sin(angle)*(radius+sqrt(25*cmp_abs(saveBuffer[j-1][i]))),z_Cir);
                angle += 4*(2*MY_PIE)/100;
                radius -= 0.005;
                
            }
            glEnd();
            
            radius = .7;
            AngleLocation += (2*MY_PIE)/12;
            //radiusLocation -= 0.1;
            z_Cir -= 0.1;
        }

    glPopMatrix();

}

//-----------------------------------------------------------------------------
// Name: fftViewGenerateFunc()
// Desc: use to generate the view of realtime fft spectrum
//-----------------------------------------------------------------------------
void fftViewGenerateFunc(complex* cbuf)
{

    // push the matrix (Lower graphic for realtime fft signal)
    glPushMatrix();
    
    glLineWidth(2.0);
    glColor3f(1, 1,0.1);
    GLfloat x_fft = 0;
    GLfloat y_fft = -3.3;
    GLfloat y_fftInc = (fabs(y_fft)*2)/(g_fbufferSize/2);
    
    // go
    glBegin( GL_LINE_STRIP );
    // loop through global buffer
    for( int i = 0; i < g_fbufferSize/2; i++ )
    {
        // set the next vertex
        glVertex2f( x_fft+sqrt(30*cmp_abs(cbuf[i])), y_fft);
        // increment x
        y_fft += y_fftInc;
    }
    // done
    glEnd();
    
    // pop
    glPopMatrix();

}

//-----------------------------------------------------------------------------
// Name: SignalViewGenerateFunc()
// Desc: use to generate the view of realtime signal graphic
//-----------------------------------------------------------------------------
void signalViewGenerateFunc(SAMPLE* buffer, int bufferSize, float colorIndex)
{

    // push the matrix (Upper graphic for realtime input signal)
    glPushMatrix();
    
    // line width
    glLineWidth( 1.5 );
    // step through and plot the waveform
    GLfloat x = -3.1;
    GLfloat y =  0;
    // increment
    GLfloat xinc = ::fabs(2*x / (bufferSize));
    
    // set the color
    glColor3f( (sin(colorIndex)+1)/2, (sin(colorIndex*2)+1)/2, (sin(colorIndex+.5)+1)/2 );
    
    // go
    glBegin( GL_LINE_STRIP );
    // loop through global buffer
    for( int i = 0; i < bufferSize; i++ )
    {
        if (x>-3 && x<3)
            // set the next vertex
            glVertex3f( x, g_xScale*buffer[i]+y,0);
        else
            glVertex3f( x, y,0);
        
        // increment x
        x += xinc;
    }
    // done
    glEnd();
    // pop
    glPopMatrix();

}

//-----------------------------------------------------------------------------
// Name: fftDataSavingFunc( )
// Desc: copy each buffer to the next then load latest fft result into the first buffer in
//       saving buffers
//-----------------------------------------------------------------------------

void fftDataSavingFunc(complex* latestBuff, int numberOfBuffer)
{
    int temp = numberOfBuffer-1;
    for (int i=0; i<temp; i++)
    {
         memcpy(saveBuffer[temp-i],saveBuffer[temp-i-1],sizeof(complex)*g_bufferSize/2);
    }
    
         memcpy(saveBuffer[0],latestBuff,sizeof(complex)*g_bufferSize/2);
}

//-----------------------------------------------------------------------------
// Name: displayFunc( )
// Desc: callback function invoked to draw the client area
//-----------------------------------------------------------------------------
void displayFunc( )
{
    // local state
    static GLfloat zrot = 0.03, c = 0.0f, r = 0.0f;
    FiltBiQuad filter;
    // clear the color and depth buffers
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    
    //display text
    GLfloat text_x = 3.5;
    GLfloat text_y = 3.5;
    textGenerateFunc(text_x,text_y+0.3,GLUT_STROKE_ROMAN,"Press ' q ' to Quit!",0.2,1.5);
    textGenerateFunc(text_x,text_y,GLUT_STROKE_ROMAN,"Press ' r ' to Rotate!",0.2,1.5);
    textGenerateFunc(text_x,text_y-0.3,GLUT_STROKE_ROMAN,"Press ' c ' to Change Mode!",0.2,1.5);
 
    //apply window to signal
    apply_window( g_buffer, g_window, g_windowSize );
    
    //copy samples to fbuffer
    memcpy( g_fbuffer, g_buffer, sizeof(SAMPLE)*g_windowSize);
    
    //transform signal into frequency domain using fft
    rfft(g_fbuffer,g_windowSize/2,FFT_FORWARD);
    
    //cast th result to be in terms of complex values
    complex* cbuf = (complex*) g_fbuffer;

///////////// Mode#1 Realtime Signal and FFT Spectrum///////////
    
    if ((program_mode%2) == 0)
    {
        
        // rotation
        //glRotatef(zrot, 0, 0, 1 );
        
        //load fft result to saving buffer
        fftDataSavingFunc(cbuf,g_numberOfSpiral);
        
        //generate realtime signal view
        signalViewGenerateFunc(g_buffer,g_bufferSize,c);
        
        //generate the view of realtime fft spectrum
        fftViewGenerateFunc(cbuf);
        
        //Generate a circle of rainbow spirals
        spiralGenerateFunc(g_numberOfSpiral,cbuf);
        

    }
    
//////////// Mode#2 Fireworks & Beat Detect///////////////////////////
    
   else
   {
       glAccum(GL_RETURN, 1.0f);

       //copy input signal to lowpassBuffer
       memcpy(g_lowpassBuffer, g_buffer, sizeof(SAMPLE)*g_lowpassBufferSize);
    
       //set filter parameters
       filter.setLP(20, 150, 1);
    
       //apply lowpass filter to input signal(g_lowpassBuffer)
       filter.process(g_lowpassBuffer,g_lowpassBuffer,g_lowpassBufferSize);

       SAMPLE avgAmp = 0;
       SAMPLE Sum = 0;
       
       for (int a=0; a<g_lowpassBufferSize; a++)
       {
           SAMPLE amplitude = fabs(g_lowpassBuffer[a]);
           Sum += amplitude;
       }
       avgAmp = Sum/g_lowpassBufferSize;
       
       if (avgAmp<0.0013)
           silent_time += 1;
       else
           music_time += 1;
       
       if ((silent_time-music_time)>40)
       {
            musicState = 0;
            music_time = 0;
           silent_time = 0;
       }
       else if((music_time-silent_time)>40)
       {
            musicState = 1;
            music_time = 0;
           silent_time = 0;
       }
       cout<<avgAmp<<endl;
       if (avgAmp>0.01 && avgAmp>(Last_avgAmp*2.1))
       {
           //g_Tempo_Period = g_time - g_Tempo_Period;
           beat = 1;
           numberOfFireworks += 1;
           int reset = (numberOfFireworks%30);
           firework[reset].newFirework();
       }
    
       Last_avgAmp = avgAmp;
       
       glPushMatrix();
            if (heartRotate == 1)
            {
                glRotatef(r,0,1,0);
                r += 0.1;
            }
            if (musicState == 1)
                drawHeartFunc(beat);
            else if (musicState == 0)
                drawStanfordFunc();
       glPopMatrix();
       
       //Display user function on screen
       textGenerateFunc(3.3,3.8,GLUT_STROKE_ROMAN,"Press ' q ' to Quit!",0.21,1.5);
       textGenerateFunc(3.3,3.5,GLUT_STROKE_ROMAN,"Press ' r ' to Rotate!",0.21,1.5);
       textGenerateFunc(3.3,3.2,GLUT_STROKE_ROMAN,"Press ' c ' to Change Mode!",0.21,1.5);
       textGenerateFunc(3.3,2.9,GLUT_STROKE_ROMAN,"Press ' h ' to Rotate Heart!",0.21,1.5);

       //Draw Fireworks
       if ((g_time%2) == 0)
       {
           glClear(GL_ACCUM_BUFFER_BIT);
           
           for (int loop=0; loop<30; loop++)
           {
               firework[loop].Move();
           }
           //Accumulate last view to make blur effect
           glAccum(GL_ACCUM, 0.9f);
       }
       g_time +=1 ;
       
   }
//////////////////////////////////////////////////////////////
         beat = 0;

    // increment color
    c += .01;
    
    // flush!
    glFlush( );
    // swap the double buffer
    glutSwapBuffers( );
}
